/*
 * Cache maintenance routines.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 1995, 1996 by Ralf Baechle and Andreas Busse
 */
#include <linux/config.h>
#include <asm/asm.h>
#include <asm/cachectl.h>
#include <asm/mipsregs.h>
#include <asm/regdef.h>
#include <asm/segment.h>

/*
 * Enable this for handling the cacheflush bug in the R4600 v1.0.  This
 * features is bad for interrupt latency and interferes with the NE2000
 * driver for unknown reasons, so I leave it off by default.
 */
#undef CONF_CACHFLUSH_BUG

/*
 * Some bits in the config register
 */
#define CONFIG_DB       (1<<4)
#define CONFIG_IB       (1<<5)

#define CACHELINES	512	/* number of cachelines (kludgy)  */

#if __mips >= 4
/*
 * Flush both ways for R10000
 */
#define CACHE(mode,addr)	cache	mode,addr;cache mode,1+addr
#else
#define CACHE(mode,addr)	cache	mode,addr
#endif

/*
 * Try to handle R4600 v1.0 cache bug without disabeling interrupts
 */
#undef CACHE
#define CACHE(mode,addr)	cache	mode,8192+addr;cache mode,addr

/*
 * Invalidate instruction/writeback and invalidate data caches.
 *
 * Parameters: a0 - starting address to flush
 *             a1 - size of area to be flushed
 *             a2 - which caches to be flushed
 *
 * FIXME:      - ignores parameters in a0/a1
 *             - doesn't know about second level caches
 *             - flushes to many lines for the 2*8kb cache R4000.
 *             - we try to handle a R4600 v1.0 CPU bug by disabeling
 *               interrupts during flushing.  Is this really ok now?
 */
		.set	noreorder
		LEAF(mips3_cacheflush)
#ifdef CONF_CACHFLUSH_BUG
		mfc0	t3,CP0_STATUS
		ori	t4,t3,1
		xori	t4,1
		mtc0	t4,CP0_STATUS
#endif
		andi	t1,a2,DCACHE
		beqz	t1,do_icache
		li	t0,KSEG0			# delay slot

#ifdef CONFIG_PROC_FS
		/*
		 * Some profiling ...
		 */
		lw	v0,dflushes
		addiu	v0,1
		sw	v0,dflushes
#endif

		/*
		 * Writeback data cache, even lines
		 */
		li	t1,CACHELINES-1
1:		CACHE	(Index_Writeback_Inv_D,0(t0))
		CACHE	(Index_Writeback_Inv_D,32(t0))
		CACHE	(Index_Writeback_Inv_D,64(t0))
		CACHE	(Index_Writeback_Inv_D,96(t0))
		CACHE	(Index_Writeback_Inv_D,128(t0))
		CACHE	(Index_Writeback_Inv_D,160(t0))
		CACHE	(Index_Writeback_Inv_D,192(t0))
		CACHE	(Index_Writeback_Inv_D,224(t0))
		CACHE	(Index_Writeback_Inv_D,256(t0))
		CACHE	(Index_Writeback_Inv_D,288(t0))
		CACHE	(Index_Writeback_Inv_D,320(t0))
		CACHE	(Index_Writeback_Inv_D,352(t0))
		CACHE	(Index_Writeback_Inv_D,384(t0))
		CACHE	(Index_Writeback_Inv_D,416(t0))
		CACHE	(Index_Writeback_Inv_D,448(t0))
		CACHE	(Index_Writeback_Inv_D,480(t0))
		addiu	t0,512
		bnez	t1,1b
		subu	t1,1

		/*
		 * Writeback data cache, odd lines
		 * Only needed for 16 byte line size
		 */
		mfc0	t1,CP0_CONFIG
		andi	t1,CONFIG_DB
		bnez	t1,do_icache
		li	t1,CACHELINES-1
1:		CACHE	(Index_Writeback_Inv_D,16(t0))
		CACHE	(Index_Writeback_Inv_D,48(t0))
		CACHE	(Index_Writeback_Inv_D,80(t0))
		CACHE	(Index_Writeback_Inv_D,112(t0))
		CACHE	(Index_Writeback_Inv_D,144(t0))
		CACHE	(Index_Writeback_Inv_D,176(t0))
		CACHE	(Index_Writeback_Inv_D,208(t0))
		CACHE	(Index_Writeback_Inv_D,240(t0))
		CACHE	(Index_Writeback_Inv_D,272(t0))
		CACHE	(Index_Writeback_Inv_D,304(t0))
		CACHE	(Index_Writeback_Inv_D,336(t0))
		CACHE	(Index_Writeback_Inv_D,368(t0))
		CACHE	(Index_Writeback_Inv_D,400(t0))
		CACHE	(Index_Writeback_Inv_D,432(t0))
		CACHE	(Index_Writeback_Inv_D,464(t0))
		CACHE	(Index_Writeback_Inv_D,496(t0))
		addiu	t0,512
		bnez	t1,1b
		subu	t1,1

do_icache:	andi	t1,a2,ICACHE
		beqz	t1,done

#ifdef CONFIG_PROC_FS
		/*
		 * Again some profiling ...
		 */
		lw	v0,iflushes
		addiu	v0,1
		sw	v0,iflushes
#endif

		/*
		 * Flush instruction cache, even lines
		 */
		lui	t0,0x8000
		li	t1,CACHELINES-1
1: 		CACHE	(Index_Invalidate_I,0(t0))
		CACHE	(Index_Invalidate_I,32(t0))
		CACHE	(Index_Invalidate_I,64(t0))
		CACHE	(Index_Invalidate_I,96(t0))
		CACHE	(Index_Invalidate_I,128(t0))
		CACHE	(Index_Invalidate_I,160(t0))
		CACHE	(Index_Invalidate_I,192(t0))
		CACHE	(Index_Invalidate_I,224(t0))
		CACHE	(Index_Invalidate_I,256(t0))
		CACHE	(Index_Invalidate_I,288(t0))
		CACHE	(Index_Invalidate_I,320(t0))
		CACHE	(Index_Invalidate_I,352(t0))
		CACHE	(Index_Invalidate_I,384(t0))
		CACHE	(Index_Invalidate_I,416(t0))
		CACHE	(Index_Invalidate_I,448(t0))
		CACHE	(Index_Invalidate_I,480(t0))
		addiu	t0,512
		bnez	t1,1b
		subu	t1,1

		/*
		 * Flush instruction cache, even lines
		 * Only needed for 16 byte line size
		 */
		mfc0	t1,CP0_CONFIG
		andi	t1,CONFIG_IB
		bnez	t1,done
		li	t1,CACHELINES-1
1:		CACHE	(Index_Invalidate_I,16(t0))
		CACHE	(Index_Invalidate_I,48(t0))
		CACHE	(Index_Invalidate_I,80(t0))
		CACHE	(Index_Invalidate_I,112(t0))
		CACHE	(Index_Invalidate_I,144(t0))
		CACHE	(Index_Invalidate_I,176(t0))
		CACHE	(Index_Invalidate_I,208(t0))
		CACHE	(Index_Invalidate_I,240(t0))
		CACHE	(Index_Invalidate_I,272(t0))
		CACHE	(Index_Invalidate_I,304(t0))
		CACHE	(Index_Invalidate_I,336(t0))
		CACHE	(Index_Invalidate_I,368(t0))
		CACHE	(Index_Invalidate_I,400(t0))
		CACHE	(Index_Invalidate_I,432(t0))
		CACHE	(Index_Invalidate_I,464(t0))
		CACHE	(Index_Invalidate_I,496(t0))
		addiu	t0,512
		bnez	t1,1b
		subu	t1,1

#ifdef CONF_CACHFLUSH_BUG
		mtc0	t3,CP0_STATUS
#endif

done:		jr	ra
		nop
		END(mips3_cacheflush)

/*
 * Invalidate virtual addresses for the floppy driver.
 *
 * Parameters: a0 - starting address to flush
 *             a1 - size of area to be flushed
 *
 * FIXME:      - ignores parameters in a0/a1
 *             - doesn't know about second level caches
 */
		.set	noreorder
		LEAF(fd_cacheflush)
		mfc0	t0,CP0_CONFIG
		andi	t0,CONFIG_DB
		srl	t0,t0,4
		li	t1,16
		sllv	t1,t1,t0		# t1 now contains line size
1:		CACHE	(Hit_Invalidate_D,0(a0))
		subu	a1,t1
		bltz	a1,1b
		addu	a0,t1			# delay slot
		jr	ra
		nop
		END(fd_cacheflush)
